@@ -1,10 +1,16 @@
 from __future__ import annotations
-from typing import Optional, Union, List, Tuple, Generic, TypeVar
-from enum import Enum
+
+import math
 from dataclasses import dataclass
+from enum import Enum
 from functools import reduce
+from typing import Generic
+from typing import List
+from typing import TypeVar
+from typing import Union
 
 import numpy as np
+import pandas as pd
 
 from . import plot
 
@@ -17,6 +23,14 @@ Image = Union[List[List[List[int]]], List[List[int]]]
 TResult = TypeVar('TResult')
 
 
+def recall_string(recall: float) -> str:
+    return 'No positive cases in GT' if math.isnan(recall) else str(recall)
+
+
+def specificity_string(specificity: float) -> str:
+    return 'No negative cases in GT' if math.isnan(specificity) else str(specificity)
+
+
 @dataclass
 class Result(Generic[TResult]):
     """Result of a GT versus Predictions
@@ -31,7 +45,7 @@ class Result(Generic[TResult]):
     """
     image: Image
     gt: TResult
-    prediction: Optional[TResult] = None
+    prediction: TResult | None = None
 
 
 class PredictionResultType(Enum):
@@ -47,94 +61,6 @@ class PredictionResult:
     type: PredictionResultType
 
 
-@dataclass
-class BBox:
-    """Bounding Box data structure
-
-    Attributes:
-        upper_left_x (int): Uper left X position of the Bounding Box.
-        upper_left_y (int): Upper left Y position of the Bounding Box.
-        width (int): Width of the Bounding Box.
-        height (int): Height of the Bounding Box.
-        cls (int): Bounding Box class index.
-        score (Optional[float]): Bounding Box prediction score.
-    """
-    upper_left_x: int
-    upper_left_y: int
-    width: int
-    height: int
-    cls: int
-    score: Optional[float] = None
-
-    @property
-    def upper_left_point(self) -> Tuple[int, int]:
-        """Tuple[int, int]: (X,Y) of the upper left point of the Bounding Box."""
-        return (self.upper_left_x, self.upper_left_y)
-
-    @property
-    def bottom_right_point(self) -> Tuple[int, int]:
-        """Tuple[int, int]: (X,Y) of the bottom right point of the Bounding Box."""
-        return (self.upper_left_x + self.width - 1, self.upper_left_y + self.height - 1)
-
-    @property
-    def center_point(self) -> Tuple[int, int]:
-        """Tuple[int, int]: (X,Y) of the center point of the Bounding Box."""
-        return ((self.upper_left_x + self.width - 1) // 2, (self.upper_left_y + self.height - 1) // 2)
-
-    @property
-    def area(self) -> int:
-        """int: Area of the Bounding Box."""
-        return self.width * self.height
-
-    def intersection_area_with(self: BBox, bbox: BBox) -> int:
-        """Calculates the intersection area with another bbox
-
-        Args:
-            self (BBox): This bbox
-            bbox (BBox): Bbox to intersect with
-
-        Returns:
-            int: The intersection area with the bbox
-        """
-
-        # Gets each box upper left and bottom right coordinates
-        (upr_lft_x_a, upr_lft_y_a) = self.upper_left_point
-        (btm_rgt_x_a, btm_rgt_y_a) = self.bottom_right_point
-
-        (upr_lft_x_b, upr_lft_y_b) = bbox.upper_left_point
-        (btm_rgt_x_b, btm_rgt_y_b) = bbox.bottom_right_point
-
-        # Calculates the intersection box upper left and bottom right coordinates
-        (upr_lft_x_intersect, upr_lft_y_intersect) = (
-            max(upr_lft_x_a, upr_lft_x_b), max(upr_lft_y_a, upr_lft_y_b))
-        (btm_rgt_x_intersect, btm_rgt_y_intersect) = (
-            min(btm_rgt_x_a, btm_rgt_x_b), min(btm_rgt_y_a, btm_rgt_y_b))
-
-        # Calculates the height and width of the intersection box
-        (w_intersect, h_intersect) = (btm_rgt_x_intersect -
-                                      upr_lft_x_intersect + 1, btm_rgt_y_intersect - upr_lft_y_intersect + 1)
-
-        # If H or W <= 0, there is no intersection
-        if (w_intersect <= 0) or (h_intersect <= 0):
-            return 0
-
-        return w_intersect * h_intersect
-
-    def union_area_with(self: BBox, bbox: BBox, intersection_area: Optional[int] = None) -> int:
-        """Calculates the union area with another bbox
-
-        Args:
-            self (BBox): This bbox
-            bbox (BBox): Bbox to union with
-            intersection_area (Optional[int], optional): The intersection area between this and bbox. Defaults to None.
-
-        Returns:
-            int: The union area with the bbox
-        """
-
-        return self.area + bbox.area - (intersection_area or self.intersection_area_with(bbox))
-
-
 @dataclass
 class Classification:
     """Represents a classification
@@ -144,7 +70,7 @@ class Classification:
         score (Optional[float], optional): Prediction score. Defaults to None.
     """
     cls: int
-    score: Optional[float] = None
+    score: float | None = None
 
 
 @dataclass
@@ -165,6 +91,11 @@ class BinaryClassificationMetrics:
     TN: int = 0
     FN: int = 0
 
+    @property
+    def has_instances(self) -> bool:
+        """int: Indicates if the class has any ground truth or predicted instances."""
+        return self.count > 0
+
     @property
     def count(self) -> int:
         """int: Total count of classified instances."""
@@ -173,35 +104,53 @@ class BinaryClassificationMetrics:
     @property
     def accuracy(self) -> float:
         """int: Total count of classified instances."""
-        return (self.TP + self.TN)/self.count
+        if self.count == 0:
+            return math.nan
+        return (self.TP + self.TN) / self.count
 
     @property
     def recall(self) -> float:
         """float: Recall metric - TP / (TP + FN)."""
-        return self.TP/(self.TP + self.FN)
+        if self.TP == 0 and self.FN == 0:
+            return math.nan
+        return self.TP / (self.TP + self.FN)
 
     @property
     def false_positive_rate(self) -> float:
         """float: False Positive Rate (FPR) metric - FP / (FP + TN)."""
-        return self.FP/(self.FP + self.TN)
+        if self.FP == 0 and self.TN == 0:
+            return math.nan
+        return self.FP / (self.FP + self.TN)
 
     @property
     def specificity(self) -> float:
         """float: Specificity metric - TN / (FP + TN)."""
-        return self.TN/(self.FP + self.TN)
+        if self.FP == 0 and self.TN == 0:
+            return math.nan
+
+        return self.TN / (self.FP + self.TN)
 
     @property
     def precision(self) -> float:
         """float: Precision metric - TP / (FP + TP)."""
-        return self.TP/(self.FP + self.TP)
+        if self.FP == 0 and self.FN == 0:  # No GT instances
+            return 1
+        elif self.FP == 0 and self.TP == 0:
+            return math.nan
+        return self.TP / (self.FP + self.TP)
 
     @property
     def f_score(self) -> float:
         """float: F-Score/Dice metric - 2*TP / (FP + FN + 2*TP)."""
-        return 2*self.TP/(self.FP + self.FN + 2*self.TP)
+        quotient = (self.FP + self.FN + 2 * self.TP)
+        if self.TP == 0 and self.FP == 0 and self.FN == 0:  # No GT instances
+            return 1
+        elif quotient == 0:
+            return math.nan
+        return 2 * self.TP / quotient
 
     @property
-    def confusion_matrix(self) -> List[List[int]]:
+    def confusion_matrix(self) -> list[list[int]]:
         """List[List[int]]: Confusion matrix of all the classes"""
         return [[self.TP, self.FP], [self.FN, self.TN]]
 
@@ -214,20 +163,38 @@ class BinaryClassificationMetrics:
         return plot.confusion_matrix(self.confusion_matrix, ['P', 'N'], f'Confusion Matrix for \"{self.cls}\" Class')
 
     def __str__(self):
+        no_instances_string = ' [NO INSTANCES]' if not self.has_instances else ''
         return (
-            f'{self.cls}:\n'
+            f'{self.cls}{no_instances_string}:\n'
             f'\tTP: {self.TP}\n'
             f'\tTN: {self.TN}\n'
             f'\tFP: {self.FP}\n'
             f'\tFN: {self.FN}\n'
             f'\tFPR: {self.false_positive_rate}\n'
             f'\tAccuracy: {self.accuracy}\n'
-            f'\tRecall: {self.recall}\n'
+            f'\tRecall: {recall_string(self.recall)}\n'
             f'\tPrecision: {self.precision}\n'
-            f'\tSpecificity: {self.specificity}\n'
+            f'\tSpecificity: {specificity_string(self.specificity)}\n'
             f'\tF-Score: {self.f_score}'
         )
 
+    def to_dict(self) -> dict:
+        if not self.has_instances:
+            return {}
+        else:
+            return {
+                'TP': self.TP,
+                'TN': self.TN,
+                'FP': self.FP,
+                'FN': self.FN,
+                'FPR': self.false_positive_rate,
+                'Accuracy': self.accuracy,
+                'Recall': recall_string(self.recall),
+                'Precision': self.precision,
+                'Specificity': specificity_string(self.specificity),
+                'F-Score': self.f_score,
+            }
+
 
 class ClassificationMetrics:
     """Multiclass classification metrics
@@ -237,7 +204,7 @@ class ClassificationMetrics:
         confusion_matrix (List[List[int]]): Confusion matrix of all the classes.
     """
 
-    def __init__(self, classes: List[str], confusion_matrix: List[List[int]] = []):
+    def __init__(self, classes: list[str], confusion_matrix: list[list[int]] = []):
         """
         Args:
             classes (List[str]): Class names.
@@ -254,13 +221,20 @@ class ClassificationMetrics:
         self._classes = classes
 
         self._by_class = self.__get_by_class_metrics()
+        self._by_class_w_instances = list(filter(
+            lambda c: c.has_instances, self._by_class))
         self._count = self._confusion_matrix.sum()
 
     @property
-    def by_class(self) -> List[BinaryClassificationMetrics]:
+    def by_class(self) -> list[BinaryClassificationMetrics]:
         """List[BinaryClassificationMetrics]: Binary metrics calculated for each class index."""
         return self._by_class
 
+    @property
+    def by_class_w_instances(self) -> list[BinaryClassificationMetrics]:
+        """List[BinaryClassificationMetrics]: Binary metrics calculated for each class index with instances."""
+        return self._by_class_w_instances
+
     @property
     def count(self) -> int:
         """int: Total count of classified instances."""
@@ -274,30 +248,40 @@ class ClassificationMetrics:
     @property
     def avg_recall(self) -> float:
         """float: Macro average recall metric."""
-        return reduce(lambda acc, curr: curr.recall + acc, self.by_class, .0) / len(self.by_class)
+        by_class_w_recall = [
+            c for c in self.by_class_w_instances if not math.isnan(c.recall)]
+
+        if (len(by_class_w_recall) == 0):
+            return 1
+        return reduce(lambda acc, curr: curr.recall + acc, by_class_w_recall, .0) / len(by_class_w_recall)
 
     @property
     def avg_precision(self) -> float:
         """float: Macro average precision metric."""
-        return reduce(lambda acc, curr: curr.precision + acc, self.by_class, .0) / len(self.by_class)
+        return reduce(lambda acc, curr: (0 if math.isnan(curr.precision)
+                                         else curr.precision) + acc,
+                      self.by_class_w_instances, .0) / len(self.by_class_w_instances)
 
     @property
     def avg_specificity(self) -> float:
         """float: Macro average specificity metric."""
-        return reduce(lambda acc, curr: curr.specificity + acc, self.by_class, .0) / len(self.by_class)
+        by_class_w_specificity = [c for c in self.by_class_w_instances
+                                  if not math.isnan(c.specificity)]
+        return reduce(lambda acc, curr: curr.specificity + acc, by_class_w_specificity, .0) / len(by_class_w_specificity)
 
     @property
     def avg_f_score(self) -> float:
         """float: Macro average F-Score/Dice metric."""
-        return reduce(lambda acc, curr: curr.f_score + acc, self.by_class, .0) / len(self.by_class)
+        return reduce(lambda acc, curr: curr.f_score + acc, self.by_class_w_instances, .0) / len(self.by_class_w_instances)
 
     @property
     def avg_false_positive_rate(self) -> float:
         """float: Macro average False Positive Rate metric."""
-        return reduce(lambda acc, curr: curr.false_positive_rate + acc, self.by_class, .0) / len(self.by_class)
+        return reduce(lambda acc, curr: curr.false_positive_rate + acc,
+                      self.by_class_w_instances, .0) / len(self.by_class_w_instances)
 
     @property
-    def confusion_matrix(self) -> List[List[int]]:
+    def confusion_matrix(self) -> list[list[int]]:
         """List[List[int]]: Confusion matrix of all the classes"""
         return self._confusion_matrix
 
@@ -305,7 +289,8 @@ class ClassificationMetrics:
         """Plots de confusion matrix
 
         Return:
-            Tuple[Figure, Axes]: Figure and Axes of the ploted confusion matrix
+            Tuple[Figure, Axes]: Figure and Axes of the plotted confusion
+            matrix
         """
         return plot.confusion_matrix(self._confusion_matrix, self._classes)
 
@@ -340,6 +325,28 @@ class ClassificationMetrics:
         ) + '\n'.join([cls_metrics.__str__()
                        for cls_metrics in self.by_class])
 
+    def to_dict(self) -> dict:
+        return {
+            'Sample Count': self.count,
+            'Average': {
+                'Accuracy': self.accuracy,
+                'Recall': self.avg_recall,
+                'Precision': self.avg_precision,
+                'Specificity': self.avg_specificity,
+                'FPR': self.avg_false_positive_rate,
+                'F-Score': self.avg_f_score
+            },
+            'By Class': dict_by_class(self.by_class)
+        }
+
+    def to_dataframe(self) -> pd.DataFrame:
+        as_dict = self.to_dict()
+        dict_to_df = {
+            'Average': as_dict['Average'],
+            **as_dict['By Class']
+        }
+        return pd.DataFrame(dict_to_df)
+
 
 class BinarySegmentationMetrics(BinaryClassificationMetrics):
     """Binary pixel-based classification metrics
@@ -354,7 +361,7 @@ class BinarySegmentationMetrics(BinaryClassificationMetrics):
     def __init__(self, classification_metrics: BinaryClassificationMetrics):
         """
         Args:
-            classification_metrics (BinaryClassificationMetrics): Pixel-based binary classification metrics 
+            classification_metrics (BinaryClassificationMetrics): Pixel-based binary classification metrics
         """
         super().__init__(
             cls=classification_metrics.cls,
@@ -367,7 +374,10 @@ class BinarySegmentationMetrics(BinaryClassificationMetrics):
     @property
     def iou(self) -> float:
         """float: IoU/Jaccard Index metric - TP / (FP + FN + TP)."""
-        return self.TP / (self.FP + self.FN + self.TP)
+        quotient = (self.FP + self.FN + self.TP)
+        if quotient == 0:
+            return math.nan
+        return self.TP / quotient
 
     def __str__(self):
         return (
@@ -378,13 +388,28 @@ class BinarySegmentationMetrics(BinaryClassificationMetrics):
             f'\tFN: {self.FN}\n'
             f'\tIoU: {self.iou}\n'
             f'\tAccuracy: {self.accuracy}\n'
-            f'\tRecall: {self.recall}\n'
+            f'\tRecall: {recall_string(self.recall)}\n'
             f'\tPrecision: {self.precision}\n'
-            f'\tSpecificity: {self.specificity}\n'
+            f'\tSpecificity: {specificity_string(self.specificity)}\n'
             f'\tFPR: {self.false_positive_rate}\n'
             f'\tF-Score: {self.f_score}\n'
         )
 
+    def to_dict(self) -> dict:
+        return {
+            'TP': self.TP,
+            'TN': self.TN,
+            'FP': self.FP,
+            'FN': self.FN,
+            'IoU': self.iou,
+            'Accuracy': self.accuracy,
+            'Recall': recall_string(self.recall),
+            'Precision': self.precision,
+            'Specificity': specificity_string(self.specificity),
+            'FPR': self.false_positive_rate,
+            'F-Score': self.f_score
+        }
+
 
 class SegmentationMetrics(ClassificationMetrics):
     """Multiclass pixel-based classification metrics
@@ -394,7 +419,7 @@ class SegmentationMetrics(ClassificationMetrics):
         confusion_matrix (List[List[int]]): Confusion matrix of all the classes.
     """
 
-    def __init__(self, classes: List[str], confusion_matrix: List[List[int]] = []):
+    def __init__(self, classes: list[str], confusion_matrix: list[list[int]] = []):
         """
         Args:
             classes (List[str]): Class names. It is expected the first class to be the background class.
@@ -408,16 +433,19 @@ class SegmentationMetrics(ClassificationMetrics):
             'There should be at least two classes (with background as the first)'
         super().__init__(classes, confusion_matrix)
         self._by_class = [BinarySegmentationMetrics(x) for x in self.by_class]
+        self._by_class_w_instances = [
+            x for x in self.by_class if x.has_instances]
 
     @property
     def avg_iou(self) -> float:
         """float: Macro average IoU/Jaccard Index metric."""
-        return reduce(lambda acc, curr: curr.iou + acc, self._by_class, .0) / len(self.by_class)
+        return reduce(lambda acc, curr: curr.iou + acc, self._by_class_w_instances, .0) / len(self._by_class_w_instances)
 
     @property
     def avg_iou_no_bkg(self) -> float:
         """float: Macro average IoU/Jaccard Index metric without `background` class (index 0)."""
-        return reduce(lambda acc, curr: curr.iou + acc, self._by_class[1:], .0) / (len(self.by_class) - 1)
+        return reduce(lambda acc, curr: curr.iou + acc,
+                      self._by_class_w_instances[1:], .0) / (len(self._by_class_w_instances) - 1)
 
     def __str__(self):
         return (
@@ -432,8 +460,32 @@ class SegmentationMetrics(ClassificationMetrics):
             f'\tAvg IoU: {self.avg_iou}\n'
             f'\tAvg IoU w/o Background: {self.avg_iou_no_bkg}\n'
             f'By Class:\n\n'
-        ) + '\n'.join(list([cls_metrics.__str__()
-                            for cls_metrics in self.by_class]))
+        ) + '\n'.join(list(cls_metrics.__str__()
+                           for cls_metrics in self.by_class))
+
+    def to_dict(self) -> dict:
+        return {
+            'Pixel Count': self.count,
+            'Average': {
+                'Accuracy': self.accuracy,
+                'Recall': self.avg_recall,
+                'Precision': self.avg_precision,
+                'Specificity': self.avg_specificity,
+                'F-Score': self.avg_f_score,
+                'FPR': self.avg_false_positive_rate,
+                'IoU': self.avg_iou,
+                'IoU w/o Background': self.avg_iou_no_bkg
+            },
+            'By Class': dict_by_class(self.by_class)
+        }
+
+    def to_dataframe(self) -> pd.DataFrame:
+        as_dict = self.to_dict()
+        dict_to_df = {
+            'Average': as_dict['Average'],
+            **as_dict['By Class']
+        }
+        return pd.DataFrame(dict_to_df)
 
 
 class BinaryDetectionMetrics(BinaryClassificationMetrics):
@@ -448,7 +500,7 @@ class BinaryDetectionMetrics(BinaryClassificationMetrics):
 
     def __init__(self, classification_metrics: BinaryClassificationMetrics,
                  iou: float,
-                 predictions: List[PredictionResult]):
+                 predictions: list[PredictionResult]):
         super().__init__(
             cls=classification_metrics.cls,
             TP=classification_metrics.TP,
@@ -457,7 +509,7 @@ class BinaryDetectionMetrics(BinaryClassificationMetrics):
         )
         self._iou = iou
         self._precision_recall_curve = self.__calculate_precision_recall_curve(
-            predictions)
+            predictions) if self.gt_count > 0 else []
 
     @property
     def gt_count(self) -> int:
@@ -478,24 +530,24 @@ class BinaryDetectionMetrics(BinaryClassificationMetrics):
         return self._iou
 
     @property
-    def precision_recall_curve(self) -> List[Tuple[float, float]]:
+    def precision_recall_curve(self) -> list[tuple[float, float]]:
         """List[Tuple[float, float]]: Precision x Recall curve as a list of (Recall, Precision) tuples."""
+        assert self.gt_count > 0, 'This class does not have instances.'
         return self._precision_recall_curve
 
-    def average_precision(self, interpolation_points: Optional[int] = None) -> float:
+    def average_precision(self, interpolation_points: int | None = None) -> float:
         """Calculates the Average Precision metric.
 
         Args:
-            interpolation_points (Optional[int], optional): Number of points to use for interpolation. 
+            interpolation_points (Optional[int], optional): Number of points to use for interpolation.
             Uses all points if None. Defaults to None.
 
         Returns:
             float: Average Precision metric.
         """
-        if not interpolation_points is None:
+        if interpolation_points is not None:
             return self._interpolated_average_precision(interpolation_points)
-        else:
-            return self._average_precision()
+        return self._average_precision()
 
     def show_precision_recall_curve(self):
         """Plots de Precision x Recall curve.
@@ -505,19 +557,22 @@ class BinaryDetectionMetrics(BinaryClassificationMetrics):
         """
         return plot.precision_recall_curve([self._precision_recall_curve], [self.cls])
 
-    def __calculate_precision_recall_curve(self, predictions: List[PredictionResult]) -> List[Tuple[float, float]]:
-        assert self.gt_count > 0, "The GT count must be greater than 0."
+    def __calculate_precision_recall_curve(self, predictions: list[PredictionResult]) -> list[tuple[float, float]]:
+        assert self.gt_count > 0, 'The GT count must be greater than 0.'
 
         sorted_predictions = sorted(
-            predictions, key=lambda p: p.score, reverse=True)
+            predictions, key=lambda p: float(p.score), reverse=True)
 
-        curve: List[Tuple[float, float]] = []
+        curve: list[tuple[float, float]] = []
         tp = 0
         fp = 0
         gt_count = self.gt_count
 
-        def calc_precision(tp, fp): return tp/(tp + fp)
-        def calc_recall(tp): return tp/gt_count
+        def calc_precision(tp, fp):
+            return tp / (tp + fp)
+
+        def calc_recall(tp):
+            return tp / gt_count
 
         for prediction in sorted_predictions:
             if prediction.type == PredictionResultType.TP:
@@ -561,13 +616,28 @@ class BinaryDetectionMetrics(BinaryClassificationMetrics):
             f'\tTN: [NA]\n'
             f'\tIoU: {self.iou}\n'
             f'\tAccuracy: {self.accuracy}\n'
-            f'\tRecall: {self.recall}\n'
+            f'\tRecall: {recall_string(self.recall)}\n'
             f'\tPrecision: {self.precision}\n'
             f'\tF-Score: {self.f_score}\n'
             f'\tAverage Precision: {self.average_precision()}\n'
             f'\t11-point Average Precision: {self.average_precision(11)}\n'
         )
 
+    def to_dict(self) -> dict:
+        return {
+            'TP': self.TP,
+            'FP': self.FP,
+            'FN': self.FN,
+            'TN': math.nan,
+            'IoU': self.iou,
+            'Accuracy': self.accuracy,
+            'Recall': recall_string(self.recall),
+            'Precision': self.precision,
+            'F-Score': self.f_score,
+            'Average Precision': self.average_precision(),
+            '11-point Average Precision': self.average_precision(11)
+        }
+
 
 class DetectionMetrics(ClassificationMetrics):
     """Multiclass detection metrics
@@ -580,14 +650,16 @@ class DetectionMetrics(ClassificationMetrics):
         The last column and line must correspond to the "undetected" class.
     """
 
-    def __init__(self, classes: List[str],
-                 confusion_matrix: List[List[int]],
-                 iou_by_class: List[float],
-                 predictions_by_class: List[List[PredictionResult]]):
+    def __init__(self, classes: list[str],
+                 confusion_matrix: list[list[int]],
+                 iou_by_class: list[float],
+                 predictions_by_class: list[list[PredictionResult]]):
         super().__init__(classes, confusion_matrix)
         by_class_wo_undetected = self.by_class[slice(0, -1)]
         self._by_class = [BinaryDetectionMetrics(by_class, iou, predictions)
                           for by_class, iou, predictions in zip(by_class_wo_undetected, iou_by_class, predictions_by_class)]
+        self._by_class_w_instances = [
+            x for x in self.by_class if x.has_instances]
 
     @property
     def avg_iou(self):
@@ -595,19 +667,21 @@ class DetectionMetrics(ClassificationMetrics):
 
         This metric is calculated as for segmentation, considering bboxes as pixel masks.
         """
-        return reduce(lambda acc, curr: curr.iou + acc, self.by_class, .0) / len(self.by_class)
+        return reduce(lambda acc, curr: curr.iou + acc, self._by_class_w_instances, .0) / len(self._by_class_w_instances)
 
-    def mean_average_precision(self, interpolation_points: Optional[int] = None) -> float:
+    def mean_average_precision(self, interpolation_points: int | None = None) -> float:
         """Calculates the mean of Average Precision metrics of all classes.
 
         Args:
-            interpolation_points (Optional[int], optional): Number of points to use for interpolation. 
+            interpolation_points (Optional[int], optional): Number of points to use for interpolation.
             Uses all points if None. Defaults to None.
 
         Returns:
             float: Mean Average Precision metric.
         """
-        return reduce(lambda acc, curr: curr.average_precision(interpolation_points) + acc, self._by_class, .0) / len(self._by_class)
+        return reduce(lambda acc,
+                      curr: curr.average_precision(interpolation_points) + acc, self._by_class_w_instances,
+                      .0) / len(self._by_class_w_instances)
 
     def __str__(self):
         return (
@@ -619,5 +693,32 @@ class DetectionMetrics(ClassificationMetrics):
             f'\tAvg F-Score: {self.avg_f_score}\n'
             f'\tAvg IoU: {self.avg_iou}\n'
             f'By Class:\n\n'
-        ) + '\n'.join(list([cls_metrics.__str__()
-                            for cls_metrics in self.by_class]))
+        ) + '\n'.join(list(cls_metrics.__str__()
+                           for cls_metrics in self.by_class))
+
+    def to_dict(self) -> dict:
+        return {
+            'Bboxes Count': self.count,
+            'Average': {
+                'Accuracy': self.accuracy,
+                'Recall': self.avg_recall,
+                'Precision': self.avg_precision,
+                'F-Score': self.avg_f_score,
+                'IoU': self.avg_iou
+            },
+            'By Class': dict_by_class(self.by_class)
+        }
+
+    def to_dataframe(self) -> pd.DataFrame:
+        as_dict = self.to_dict()
+        dict_to_df = {
+            'Average': as_dict['Average'],
+            **as_dict['By Class']
+        }
+        return pd.DataFrame(dict_to_df)
+
+
+def dict_by_class(by_class) -> dict:
+    return {
+        cls_metrics.cls: cls_metrics.to_dict() for cls_metrics in by_class
+    }
